/**
 * @author Toru Nagashima
 * See LICENSE file in root directory for full license.
 */
"use strict"

require("./update-docs-header-and-footer")
require("./update-lib-index")
require("./update-readme")
